@@ -184,10 +184,10 @@ class FaultHandlerTests(unittest.TestCase):
         self.check_fatal_error("""
             import faulthandler
             faulthandler.enable()
-            faulthandler._read_null(True)
+            faulthandler._sigsegv(True)
             """,
             3,
-            '(?:Segmentation fault|Bus error|Illegal instruction)')
+            'Segmentation fault')
 
     def test_enable_file(self):
         with temporary_filename() as filename:
@@ -220,7 +220,7 @@ class FaultHandlerTests(unittest.TestCase):
             """
         not_expected = 'Fatal Python error'
         stderr, exitcode = self.get_output(code)
-        stder = '\n'.join(stderr)
+        stderr = '\n'.join(stderr)
         self.assertTrue(not_expected not in stderr,
                      "%r is present in %r" % (not_expected, stderr))
         self.assertNotEqual(exitcode, 0)
@@ -250,17 +250,25 @@ class FaultHandlerTests(unittest.TestCase):
     def test_disabled_by_default(self):
         # By default, the module should be disabled
         code = "import faulthandler; print(faulthandler.is_enabled())"
-        args = (sys.executable, '-E', '-c', code)
-        # don't use assert_python_ok() because it always enable faulthandler
-        output = subprocess.check_output(args)
+        args = filter(None, (sys.executable,
+                             "-E" if sys.flags.ignore_environment else "",
+                             "-c", code))
+        env = os.environ.copy()
+        env.pop("PYTHONFAULTHANDLER", None)
+        # don't use assert_python_ok() because it always enables faulthandler
+        output = subprocess.check_output(args, env=env)
         self.assertEqual(output.rstrip(), b"False")
 
     def test_sys_xoptions(self):
         # Test python -X faulthandler
         code = "import faulthandler; print(faulthandler.is_enabled())"
-        args = (sys.executable, "-E", "-X", "faulthandler", "-c", code)
-        # don't use assert_python_ok() because it always enable faulthandler
-        output = subprocess.check_output(args)
+        args = filter(None, (sys.executable,
+                             "-E" if sys.flags.ignore_environment else "",
+                             "-X", "faulthandler", "-c", code))
+        env = os.environ.copy()
+        env.pop("PYTHONFAULTHANDLER", None)
+        # don't use assert_python_ok() because it always enables faulthandler
+        output = subprocess.check_output(args, env=env)
         self.assertEqual(output.rstrip(), b"True")
 
     def test_env_var(self):
@@ -269,7 +277,7 @@ class FaultHandlerTests(unittest.TestCase):
         args = (sys.executable, "-c", code)
         env = os.environ.copy()
         env['PYTHONFAULTHANDLER'] = ''
-        # don't use assert_python_ok() because it always enable faulthandler
+        # don't use assert_python_ok() because it always enables faulthandler
         output = subprocess.check_output(args, env=env)
         self.assertEqual(output.rstrip(), b"False")
 
