@@ -7,10 +7,6 @@ import socket
 import urllib.error
 import urllib.request
 import sys
-try:
-    import ssl
-except ImportError:
-    ssl = None
 
 support.requires("network")
 
@@ -106,8 +102,6 @@ class OtherNetworkTests(unittest.TestCase):
             'ftp://ftp.debian.org/debian/README',
             ('ftp://ftp.debian.org/debian/non-existent-file',
              None, urllib.error.URLError),
-            'ftp://gatekeeper.research.compaq.com/pub/DEC/SRC'
-                '/research-reports/00README-Legal-Rules-Regs',
             ]
         self._test_urls(urls, self._extra_handlers())
 
@@ -157,20 +151,20 @@ class OtherNetworkTests(unittest.TestCase):
 ##             self._test_urls(urls, self._extra_handlers()+[bauth, dauth])
 
     def test_urlwithfrag(self):
-        urlwith_frag = "https://docs.python.org/2/glossary.html#glossary"
+        urlwith_frag = "http://www.pythontest.net/index.html#frag"
         with support.transient_internet(urlwith_frag):
             req = urllib.request.Request(urlwith_frag)
             res = urllib.request.urlopen(req)
             self.assertEqual(res.geturl(),
-                    "https://docs.python.org/2/glossary.html#glossary")
+                    "http://www.pythontest.net/index.html#frag")
 
     def test_redirect_url_withfrag(self):
-        redirect_url_with_frag = "http://bit.ly/1iSHToT"
+        redirect_url_with_frag = "http://www.pythontest.net/redir/with_frag/"
         with support.transient_internet(redirect_url_with_frag):
             req = urllib.request.Request(redirect_url_with_frag)
             res = urllib.request.urlopen(req)
             self.assertEqual(res.geturl(),
-                    "https://docs.python.org/3.4/glossary.html#term-global-interpreter-lock")
+                    "http://www.pythontest.net/elsewhere/#frag")
 
     def test_custom_headers(self):
         url = "http://www.example.com"
@@ -224,6 +218,7 @@ class OtherNetworkTests(unittest.TestCase):
                 with support.transient_internet(url):
                     try:
                         f = urlopen(url, req, TIMEOUT)
+                    # urllib.error.URLError is a subclass of OSError
                     except OSError as err:
                         if expected_err:
                             msg = ("Didn't get expected error(s) %s for %s %s, got %s: %s" %
@@ -231,12 +226,6 @@ class OtherNetworkTests(unittest.TestCase):
                             self.assertIsInstance(err, expected_err, msg)
                         else:
                             raise
-                    except urllib.error.URLError as err:
-                        if isinstance(err[0], socket.timeout):
-                            print("<timeout: %s>" % url, file=sys.stderr)
-                            continue
-                        else:
-                            raise
                     else:
                         try:
                             with support.time_out, \
@@ -300,7 +289,7 @@ class TimeoutTest(unittest.TestCase):
             self.addCleanup(u.close)
             self.assertEqual(u.fp.raw._sock.gettimeout(), 120)
 
-    FTP_HOST = "ftp://ftp.mirror.nl/pub/gnu/"
+    FTP_HOST = 'ftp://ftp.debian.org/debian/'
 
     def test_ftp_basic(self):
         self.assertIsNone(socket.getdefaulttimeout())
